use crate::config;
use core::fmt;
use serde::Deserialize;

#[derive(Deserialize, Debug)]
pub struct MinecraftVersion {
    pub version: String,
    pub version_type: String,
    pub date: String,
    pub major: bool,
}

#[derive(Deserialize, Debug)]
pub struct SearchProject {
    pub title: String,
    pub description: String,
    pub project_id: String,
    pub game_versions: Vec<String>,
    pub versions: Vec<String>,
}

impl fmt::Display for SearchProject {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        write!(f, "{}", self.title)
    }
}

#[derive(Deserialize, Debug)]
pub struct Project {
    pub title: String,
    pub description: String,
    pub id: String,
    pub game_versions: Vec<String>,
    pub versions: Vec<String>,
}

impl fmt::Display for Project {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        write!(f, "{}", self.title)
    }
}

#[derive(Deserialize, Debug)]
pub struct Search {
    hits: Vec<SearchProject>,
}

#[derive(Deserialize, Debug)]
pub struct ProjectVersion {
    pub name: String,
    pub game_versions: Vec<String>,
    pub version_type: String,
    pub id: String,
    pub project_id: String,
}

pub async fn get_minecraft_versions() -> Result<Vec<MinecraftVersion>, reqwest::Error> {
    reqwest::get("https://api.modrinth.com/v2/tag/game_version")
        .await?
        .json::<Vec<MinecraftVersion>>()
        .await
}

pub async fn search_projects(
    query: String,
    version: &String,
    loader: &String,
) -> Result<Vec<SearchProject>, reqwest::Error> {
    let url = format!("https://api.modrinth.com/v2/search?query={query}&facets=[[\"project_type:mod\"],[\"versions:{version}\"],[\"categories:{loader}\"]]");

    let response = reqwest::get(url).await?.json::<Search>().await?;

    Ok(response.hits)
}

pub async fn get_project_versions(
    id: &String,
    version: &String,
    loader: &String,
) -> Result<Vec<ProjectVersion>, reqwest::Error> {
    let url = format!("https://api.modrinth.com/v2/project/{id}/version?loaders=[\"{loader}\"]&game_versions=[\"{version}\"]");

    let response = reqwest::get(url)
        .await?
        .json::<Vec<ProjectVersion>>()
        .await?;

    Ok(response)
}

pub async fn get_multiple_project(
    project_ids: Vec<String>,
) -> Result<Vec<Project>, reqwest::Error> {
    let ids = project_ids.join("\", \"");

    let url = format!("https://api.modrinth.com/v2/projects?ids=[\"{ids}\"]");

    let response = reqwest::get(url).await?.json::<Vec<Project>>().await?;

    Ok(response)
}
